@@ -312,7 +312,9 @@ python_config <- function(python, required_module, python_versions, forced = NUL
     stop("Error ", status, " occurred running ", python, " ", errmsg)
   }
 
-  config <- read.dcf(textConnection(config), all = TRUE)
+  config_connection <- textConnection(config)
+  on.exit(close(config_connection))
+  config <- read.dcf(config_connection, all = TRUE)
 
   # get the full textual version and the numeric version, check for anaconda
   version_string <- config$Version
@@ -333,23 +335,19 @@ python_config <- function(python, required_module, python_versions, forced = NUL
     python_libdir_config <- function(var) {
       python_libdir <- config[[var]]
       ext <- switch(Sys.info()[["sysname"]], Darwin = ".dylib", Windows = ".dll", ".so")
-      libpython <- file.path(python_libdir, paste0("libpython" , version, c("", "m"), ext))
-      libpython_exists <- libpython[file.exists(libpython)]
-      if (length(libpython_exists) > 0)
-        libpython_exists[[1]]
-      else
-        libpython[[1]]
+      pattern <- paste0("^libpython", version, "m?", ext)
+      libpython <- list.files(python_libdir, pattern = pattern, full.names = TRUE)
     }
-    if (!is.null(config$LIBPL)) {
-      libpython <- python_libdir_config("LIBPL")
-      if (!file.exists(libpython)) {
-        if (!is.null(config$LIBDIR))
-          libpython <- python_libdir_config("LIBDIR")
+    for (libsrc in c("LIBPL", "LIBDIR")) {
+      if (!is.null(config[[libsrc]])) {
+        libpython <- python_libdir_config(libsrc)
+        if (length(libpython))
+          break
         else
           libpython <- NULL
+      } else {
+        libpython <- NULL
       }
-    } else {
-      libpython <- NULL
     }
   }
 
@@ -375,6 +373,13 @@ python_config <- function(python, required_module, python_versions, forced = NUL
   else
     numpy <- NULL
 
+  # check to see if this is a Python virtualenv
+  root <- dirname(dirname(python))
+  virtualenv <- if (is_python_virtualenv(root))
+    root
+  else
+    ""
+
   # check for virtualenv activate script
   activate_this <- file.path(dirname(python), "activate_this.py")
   if (file.exists(activate_this))
@@ -388,8 +393,9 @@ python_config <- function(python, required_module, python_versions, forced = NUL
   # return config info
   structure(class = "py_config", list(
     python = python,
-    libpython = libpython,
+    libpython = libpython[1],
     pythonhome = pythonhome,
+    virtualenv = virtualenv,
     virtualenv_activate = virtualenv_activate,
     version_string = version_string,
     version = version,
@@ -473,6 +479,18 @@ is_ubuntu <- function() {
   }
 }
 
+is_rstudio <- function() {
+  exists("RStudio.Version", envir = globalenv())
+}
+
+is_rstudio_desktop <- function() {
+  if (!exists("RStudio.Version", envir = globalenv()))
+    return(FALSE)
+
+  RStudio.Version <- get("RStudio.Version", envir = globalenv())
+  version <- RStudio.Version()
+  identical(version$mode, "desktop")
+}
 
 clean_version <- function(version) {
   gsub("\\.$", "", gsub("[A-Za-z_+].*$", "", version))
