@@ -1,5 +1,14 @@
 context("helper handlers")
 
+test_that("Function arguments from builtin functions can be extracted", {
+  skip_if_no_python()
+  skip_on_os("windows")
+  os <- import("os")
+  output <- help_formals_handler.python.builtin.object("chmod", os)
+  expected <- list(formals = c("path", "mode"), helpHandler = "reticulate:::help_handler")
+  expect_identical(output, expected)
+})
+
 test_that("Documentations in Sphinx style can be extracted correctly for help handlers", {
   skip_if_no_docutils()
 
@@ -26,10 +35,10 @@ Just a placeholder here.
 "
 
   doctree <- sphinx_doctree_from_doc(docs)
-  expect_equal(names(doctree$ids), c("parameters", "returns", "section1"))
+  expect_setequal(names(doctree$ids), c("parameters", "returns", "section1"))
 
   arg_descriptions <- arg_descriptions_from_doc_sphinx(docs)
-  expect_equal(names(arg_descriptions), c("a", "type"))
+  expect_setequal(names(arg_descriptions), c("a", "type"))
   expect_match(arg_descriptions[["a"]], "Description for a")
   expect_match(arg_descriptions[["type"]], "Type of algorithm")
 
@@ -39,6 +48,7 @@ Just a placeholder here.
 })
 
 test_that("Documentations in Google style can be extracted correctly for help handlers", {
+  skip_if_no_docutils()
 
   docs <- "
 Initialize the model.
